//===============================================================================
// Goal Book.
// Copyright  2009 Mark Brownsword. 
//===============================================================================

#region Using Statements
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using GoalBook.Infrastructure.Enums;
using GoalBook.Infrastructure.Events;
using GoalBook.Infrastructure.Interfaces;
using GoalBook.Infrastructure.ObjectModel;
using Infragistics.Windows.DataPresenter;
using Infragistics.Windows.DataPresenter.Events;
using Microsoft.Practices.Composite;
#endregion

namespace GoalBook.Goals.Views
{
    /// <summary>
    /// Interaction logic for GoalsView.xaml
    /// </summary>    
    public partial class GoalsView : UserControl, IView, IActiveAware
    {
        #region Constants and Enums
        #endregion

        #region Inner Classes and Structures
        #endregion

        #region Delegates and Events
        /// <summary>
        /// Raise the IsActiveChanged event.
        /// </summary>
        private void OnIsActiveChanged()
        {            
            EventHandler isActiveChangedHandler = IsActiveChanged;
            if (isActiveChangedHandler != null) isActiveChangedHandler(this, EventArgs.Empty);
        }        
        /// <summary>
        /// Raise the Activated event.
        /// </summary>
        private void OnActivated()
        {
            EventHandler activatedHandler = Activated;
            if (activatedHandler != null) activatedHandler(this, EventArgs.Empty);
        }        
        /// <summary>
        /// Raise the EditActionInitiated event.
        /// </summary>
        private void OnEditActionInitiated(SelectedRecordCollection records, EditActionType editAction)
        {            
            EditActionEventHandler editActionInitiatedHandler = EditActionInitiated;
            if (editActionInitiatedHandler != null) 
            {
                editActionInitiatedHandler(new EditActionEventArgs(records, editAction)); 
            }
        }
        /// <summary>
        /// Raise the GoalSelected event.
        /// </summary>
        private void OnGoalSelected()
        {
            EventHandler goalSelectedHandler = GoalSelected;
            if (goalSelectedHandler != null) goalSelectedHandler(this, EventArgs.Empty);
        }

        public event EventHandler GoalSelected;
        public event EventHandler IsActiveChanged;
        public event EventHandler Activated;        
        public event EditActionEventHandler EditActionInitiated;
        #endregion

        #region Instance and Shared Fields
        private bool _isActive;

        /// <summary>
        /// Declaration for isFirstload.
        /// </summary>
        private bool isFirstload;
        #endregion

        #region Constructors
        /// <summary>
        /// Constructor.
        /// </summary>
        public GoalsView()
        {
            InitializeComponent();

            this.isFirstload = true;
        }
        #endregion

        #region Properties
        /// <summary>
        /// Count of Selected Goals.
        /// </summary>        
        public int SelectedGoalsCount
        {
            get { return xamDataGridGoals.SelectedItems.Records.Count; }
        }
        /// <summary>
        /// Selected Goals.
        /// </summary>
        public SelectedRecordCollection SelectedGoals
        {
            get { return xamDataGridGoals.SelectedItems.Records; }
        }
        #endregion

        #region Private and Protected Methods
        #endregion

        #region Public and internal Methods
        /// <summary>
        /// Refresh Sort Position.
        /// </summary>
        public void RefreshSortPosition(bool activeOnly)
        {
            if (activeOnly)
            {
                if (xamDataGridGoals.ActiveRecord == null)
                {
                    return;
                }

                xamDataGridGoals.ActiveRecord.RefreshSortPosition();
            }
            else
            {                
                xamDataGridGoals.Records.RefreshSort();
            }            
        }
        /// <summary>
        /// Set Item Active.
        /// </summary>        
        public void SetItemActive(Goal goal)
        {
            if (xamDataGridGoals.Records.Count == 0)
            {
                return;
            }

            foreach (Record record in xamDataGridGoals.Records)
            {
                if (record is DataRecord)
                {
                    DataRecord dataRecord = record as DataRecord;
                    if (dataRecord.DataItem is Goal)
                    {
                        if ((dataRecord.DataItem as Goal).GoalID == goal.GoalID)
                        {
                            xamDataGridGoals.Focus();
                            xamDataGridGoals.ActiveRecord = record;
                            xamDataGridGoals.ActiveRecord.IsSelected = true;
                        }
                    }
                }
            }
        }
        #endregion

        #region Event Handlers
        /// <summary>
        /// Handle UserControl_Loaded event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            if (xamDataGridGoals.Records.Count > 0 && this.isFirstload)
            {
                xamDataGridGoals.Focus();
                xamDataGridGoals.ActiveRecord = xamDataGridGoals.Records[0];
                xamDataGridGoals.ActiveRecord.IsSelected = true;
                this.isFirstload = false;
            }            
        }

        /// <summary>
        /// Handle XamDataGridGoals_SelectedItemsChanged event.
        /// </summary>        
        private void XamDataGridGoals_SelectedItemsChanged(object sender, SelectedItemsChangedEventArgs e)
        {
            OnGoalSelected();
        }
        /// <summary>
        /// Handle XamDataGridGoals_RecordsDeleting event.
        /// </summary>        
        private void XamDataGridGoals_RecordsDeleting(object sender, RecordsDeletingEventArgs e)
        {
            e.DisplayPromptMessage = false;
            e.Cancel = true;
        }
        /// <summary>
        /// Handle XamDataGridGoals_KeyUp event.
        /// </summary>        
        private void XamDataGridGoals_KeyUp(object sender, KeyEventArgs e)
        {
            if (xamDataGridGoals.SelectedItems.Records.Count == 0) { return; }
            if (e.Key == Key.Enter && e.KeyboardDevice.Modifiers == ModifierKeys.None)
            {
                OnEditActionInitiated(xamDataGridGoals.SelectedItems.Records, EditActionType.Edit);
            }                        
        }
        /// <summary>
        /// Handle XamDataGridGoals_MouseDoubleClick event.
        /// </summary>        
        private void XamDataGridGoals_MouseDoubleClick(object sender, MouseButtonEventArgs e)
        {
            if (xamDataGridGoals.SelectedItems.Records.Count == 0) { return; }
            
            OnEditActionInitiated(xamDataGridGoals.SelectedItems.Records, EditActionType.Edit);
        }
        #endregion

        #region Base Class Overrides
        #endregion
           
        #region IView Members
        /// <summary>
        /// Activate.
        /// </summary>
        public void Activate()
        {
            Mouse.SetCursor(Cursors.Wait);
            try
            {                
                OnActivated();
            }
            finally { Mouse.UpdateCursor(); }
        }
        #endregion

        #region IActiveAware Members
        /// <summary>
        /// Reference to IsActive. Note that Global Commands get
        /// their IsActive property set here.
        /// </summary>
        public bool IsActive
        {
            get { return _isActive; }
            set
            {
                if (_isActive != value)
                {
                    _isActive = value;
                    OnIsActiveChanged();
                }
            }
        }
        #endregion        
    }
}
