// <copyright file="TasksActionView.xaml.cs" company="GoalBook"> 
//    Copyright  2009 Mark Brownsword. All rights reserved.
//    This source code and supporting files are licensed under The Code Project  
//    Open License (CPOL) as detailed at http://www.codeproject.com/info/cpol10.aspx. 
// </copyright>
namespace GoalBook.Tasks.Views
{
    #region Using Statements
    using System;
    using System.Text;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Input;
    using GoalBook.Infrastructure;
    using GoalBook.Infrastructure.Interfaces;
    using Microsoft.Practices.Composite;
    #endregion

    /// <summary>
    /// Interaction logic for TasksActionView.xaml
    /// </summary>    
    public partial class TasksActionView : UserControl, IView, IActiveAware
    {
        #region Instance and Shared Fields
        /// <summary>
        /// Declaration for isActive.
        /// </summary>
        private bool isActive;
        #endregion
        
        #region Inner Classes and Structures
        #endregion
                
        #region Constructors
        /// <summary>
        /// Initializes a new instance of the TasksActionView class.
        /// </summary>
        public TasksActionView()
        {
            InitializeComponent();
                        
            this.FolderFilter = string.Empty;
            this.CalendarFilter = CalendarFilterOption.All;            
            this.ControlsLoaded = false;
        }
        #endregion

        #region Delegates and Events
        /// <summary>
        /// IsActiveChanged EventHandler.
        /// </summary>                
        public event EventHandler IsActiveChanged;

        /// <summary>
        /// Activated EventHandler
        /// </summary>
        public event EventHandler Activated;

        /// <summary>
        /// FilterChanged EventHandler.
        /// </summary>
        public event EventHandler FilterChanged;
        #endregion

        #region Enums
        /// <summary>
        /// Calendar Filter Option.
        /// </summary>
        public enum CalendarFilterOption
        {
            /// <summary>
            /// All Tasks.
            /// </summary>
            All,

            /// <summary>
            /// Tasks with No Date.
            /// </summary>
            NoDate,

            /// <summary>
            /// Past Tasks.
            /// </summary>
            Past,

            /// <summary>
            /// Tasks for Today.
            /// </summary>
            Today,

            /// <summary>
            /// Tasks for this week.
            /// </summary>
            ThisWeek,

            /// <summary>
            /// Future Tasks.
            /// </summary>
            Future
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets or sets FolderFilter.
        /// </summary>
        public string FolderFilter { get; set; }

        /// <summary>
        /// Gets or sets CalendarFilter.
        /// </summary>
        public CalendarFilterOption CalendarFilter { get; set; }

        /// <summary>
        /// Gets or sets a value indicating whether to ListenForListChangedEvents.
        /// </summary>
        public bool ListenForListChangedEvents
        {
            get { return this.checkedListControl.RaiseCheckedItemsChangedEvent; }
            set { this.checkedListControl.RaiseCheckedItemsChangedEvent = value; }
        }
                                
        #region IActiveAware Members
        /// <summary>
        /// Gets or sets a value indicating whether IsActive. Note that Global Commands get
        /// their IsActive property set here.
        /// </summary>
        public bool IsActive
        {
            get 
            { 
                return this.isActive; 
            }

            set
            {
                if (this.isActive != value)
                {
                    this.isActive = value;
                    this.OnIsActiveChanged();
                }
            }
        }

        /// <summary>
        /// Gets or sets a value indicating whether the control has loaded.
        /// </summary>
        private bool ControlsLoaded { get; set; }
        #endregion

        #endregion

        #region Public and internal Methods
        
        #region IView Members
        /// <summary>
        /// Activate the view.
        /// </summary>
        public void Activate()
        {
            Mouse.SetCursor(Cursors.Wait);
            try
            {
                this.OnActivated();
            }
            finally
            {
                Mouse.UpdateCursor();
            }
        }
        #endregion     

        #endregion

        #region Private and Protected Methods
        /// <summary>
        /// Raise the IsActiveChanged event.
        /// </summary>
        private void OnIsActiveChanged()
        {
            EventHandler isActiveChangedHandler = this.IsActiveChanged;
            if (isActiveChangedHandler != null)
            {   
                isActiveChangedHandler(this, EventArgs.Empty); 
            }
        }

        /// <summary>
        /// Raise the Activated event.
        /// </summary>
        private void OnActivated()
        {
            EventHandler activatedHandler = this.Activated;
            if (activatedHandler != null)
            {
                activatedHandler(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Raise the FilterChanged event.
        /// </summary>
        private void OnFilterChanged()
        {
            if (this.ControlsLoaded)
            {
                EventHandler filterChangedHandler = this.FilterChanged;
                if (filterChangedHandler != null)
                {
                    filterChangedHandler(this, EventArgs.Empty);
                }
            }
        }

        /// <summary>
        /// Get Checked Items From Filter.
        /// </summary>
        /// <returns>List of checked items</returns>
        private KeyValueItemList GetCheckedItemsFromFilter()
        {
            KeyValueItemList items = new KeyValueItemList();

            foreach (KeyValueItem listItem in (this.DataContext as TasksViewPresentationModel).FolderList)
            {
                if (string.IsNullOrEmpty(this.FolderFilter) || this.FolderFilter.Contains(listItem.Key))
                {
                    items.Add(new KeyValueItem(listItem.Key, listItem.Value, listItem.Order));
                }
            }

            return items;
        }

        /// <summary>
        /// Build Filter.
        /// </summary>        
        private void BuildFolderFilter()
        {
            const char Separator = ',';
            StringBuilder builder = new StringBuilder();

            foreach (KeyValueItem content in this.checkedListControl.CheckedItems)
            {
                builder.Append(content.Key);
                builder.Append(Separator);
            }

            this.FolderFilter = builder.ToString().TrimEnd(Separator);
        }
        #endregion

        #region Event Handlers 
        /// <summary>
        /// Handle UserControl_Loaded event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {            
            this.checkedListControl.CheckedItems = this.GetCheckedItemsFromFilter();
            
            switch (this.CalendarFilter)
            {
                case CalendarFilterOption.All:
                    calendarRadioAll.IsChecked = true;
                    break;
                case CalendarFilterOption.NoDate:
                    calendarRadioNoDate.IsChecked = true;
                    break;
                case CalendarFilterOption.Past:
                    calendarRadioPast.IsChecked = true;
                    break;
                case CalendarFilterOption.Today:
                    calendarRadioToday.IsChecked = true;
                    break;
                case CalendarFilterOption.ThisWeek:
                    calendarRadioThisWeek.IsChecked = true;
                    break;
                case CalendarFilterOption.Future:
                    calendarRadioFuture.IsChecked = true;
                    break;
                default:
                    break;
            }
            
            this.BuildFolderFilter();            
            this.ControlsLoaded = true;
        }

        /// <summary>
        /// Handle CheckedListControl_CheckedItemsChanged event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void CheckedListControl_CheckedItemsChanged(object sender, RoutedEventArgs e)
        {
            this.BuildFolderFilter();
            this.OnFilterChanged();
        }

        /// <summary>
        /// Handle CheckedListControl_StatusChanged event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void CheckedListControl_StatusChanged(object sender, RoutedEventArgs e)
        {
            // When new items are added to the FolderList the filter is updated (in TasksViewPresenter.Folders_ListChanged)
            // so this event gives us a chance to update the items we want to appear as checked.            
            this.checkedListControl.CheckedItems = this.GetCheckedItemsFromFilter();
        }

        /// <summary>
        /// Handle CalendarRadioList_CheckedItemChanged event.
        /// </summary>
        /// <param name="sender">sender parameter</param>
        /// <param name="e">RoutedEventArgs parameter</param>
        private void CalendarRadioList_CheckedItemChanged(object sender, RoutedEventArgs e)
        {
            RadioButton button = e.Source as RadioButton;

            if (button.Name == calendarRadioAll.Name)
            {
                this.CalendarFilter = CalendarFilterOption.All;
            }
            else if (button.Name == calendarRadioNoDate.Name)
            {
                this.CalendarFilter = CalendarFilterOption.NoDate;
            }
            else if (button.Name == calendarRadioPast.Name)
            {
                this.CalendarFilter = CalendarFilterOption.Past;
            }
            else if (button.Name == calendarRadioToday.Name)
            {
                this.CalendarFilter = CalendarFilterOption.Today;
            }
            else if (button.Name == calendarRadioThisWeek.Name)
            {
                this.CalendarFilter = CalendarFilterOption.ThisWeek;
            }
            else if (button.Name == calendarRadioFuture.Name)
            {
                this.CalendarFilter = CalendarFilterOption.Future;
            }

            this.OnFilterChanged();
        }
        #endregion

        #region Base Class Overrides
        #endregion            
    }
}
